#include <fstream.h>
#include <string.h>
#include <ctype.h>

static char *version_number="3.8.";         // version number of the program
static char datfile[13]="dfcoderN.dat";     // name of a .DAT file
static char datfile_number='0';             // number of a .DAT file; default=0
static char delim_char;                     // delimiting character for .DAT file
static char ch, ch1=1;	                    // auxiliary characters
static const char incorr_switch[]="\n\tIncorrect switch. Must be /0, /1, ..., /9. "
                                  "Default is /0.\n";
                                            // error message

static char outfile[127];
static char infile[127];
static char exe_path_string[127];

static int datfile_line_num=1;              // counter for the number of line in .DAt file
static int counter;                         // auxiliary counter
static const int max=1+20;                  // 1+maximum length of string-substitution
static int dict_len[256];                   // array of subst-strings' lengths
static char dictionary[256][max];           // array of strings-substitutions

//===================Prototypes========================================

void coding(ifstream* fin, ofstream* fout);
                                            // "coding" function prototype
int ac(char any_char);
                                            // "ascii-code" function prototype
void dfc_ext(void);
                                            // "making .dfc extension" function prototype

void exe_path(char *ptr);                   // "making exe-path" function prototype

//===================Beginning of MAIN==================================

int main(int argc, char* argv[])
{

cout<<"\nDFcoder (file coding utility). Version " << version_number
    << " Copyright (c) D.Fomin, 1994\n";

// Processing number of parameters and HELP info

if (argc == 1)	{
cout << "\t SYNTAX: dfcoder <file.in> [<file.out>] [/<n>]\n"
    "DFcoder substitutes characters from <file.in> according to settings\n"
    "in file DFCODER<n>.DAT. Modified file is written to <file.out>. See\n"
    "detailed instructions in DFCODER.DOC.\n";
    return 0;     };

strcpy(infile, argv[1]);
   
if (argc > 2)
         {
         if (argc > 3)
		 {
                if (*(argv[3])=='/' && isdigit(*(argv[3]+1)));
		        else
			    {cout << incorr_switch;
                 return 254;
		        };
		        datfile_number=*(argv[3]+1);
                strcpy(outfile, argv[2]);
		 }
         else {
               if (*(argv[2])=='/')
                     {
                     if (isdigit(*(argv[2]+1)))
                           {
                           datfile_number=*(argv[2]+1);
                           strcpy(outfile, argv[1]);
                           dfc_ext();
                           }
                     else
                     {cout << incorr_switch;
                      return 254;
		             };
                     }
               else
               strcpy(outfile, argv[2]);
               }
         }

else    {
        strcpy(outfile, argv[1]);
        dfc_ext();
        };

//Input and output files' names must differ!

if (!strcmp(infile, outfile))
{cerr << "\n\tFatal error! Input and output filenames must differ!\n"; return 253;};

// Making .DAT file name

datfile[7]=datfile_number;

// Two main streams: ifstreams for input file and for .DAT file; stream
// for output file will be created later.
// Then check ifstreams do exist and .DAT file layout is correct

ifstream input_file;
input_file.open(infile, ios::binary);
if (!input_file) {cerr << "\n\tCannot open input file: " << infile << "\n"; return 252;};
ifstream data_file;
data_file.open(datfile, ios::binary);
if (!data_file)
   {
    cerr << "\n\tCannot open .DAT file <" << datfile
         << "> in working directory.\n\tTrying to find another one in a directory"
            " where DFCODER.EXE\n\tis located...\n"; 
    exe_path(argv[0]);
    data_file.open(exe_path_string, ios::binary);
    if (!data_file)
       {
        cerr << "\n\t\Sorry. Cannot find specified .DAT file";
        return 251;
       };
    };

// Setting delimiter

ch1=1;

do
{
    if (data_file.eof())
       {ch1=0; break;};
    data_file.get(ch);
    if (ch=='\n') datfile_line_num++;
}
while (ch!='=');

if (ch1)
   data_file.get(delim_char);
else
   {
    cerr << "\n\tIncorrect .DAT file\n\tCannot find delimiter definition\n";
    return 250;
   }

// Filling in DICTIONARY array of strings and
// DICT_LEN array of strings' lengths

for (counter=0; counter < 256; counter++)
        {dict_len[counter]=1;
         dictionary[counter][0]=(char)counter;
         dictionary[counter][1]='\0';}

while (data_file)
	{data_file.get(ch);
		if (ch!='\n');
		else
		{
         datfile_line_num++;
         data_file.get(ch);
         if (data_file.eof()) break;
         data_file.get(ch1);
         if (ch1!=delim_char)
                  {
                  cerr << "\n\tIncorrect .DAT file\n\tSee line "
                       << datfile_line_num <<" or delimiter definition\n";
                  return 255;
                  };
		 data_file.get(dictionary[ac(ch)], max, delim_char);
		 dict_len[ac(ch)]=strlen(dictionary[ac(ch)]);
		};
	};

// Creating ofstream for output file. Check its existence

ofstream output_file;
output_file.open(outfile, ios::binary);
if (!output_file) {cerr << "\n\tCannot open output file: " << outfile << "\n"; return 249;};

// Call of CODING function

coding(&input_file, &output_file);
cout << "\n\tSuccess!\n";
return 0;
}

//=================End of MAIN============================================

// Definition of CODING function
                    
void coding(ifstream* fin, ofstream* fout)
     {
unsigned char curr_char;	// current char to be coded

// take current input char from input file (cycle through the end of input)
while(fin->get(curr_char))
// write substitution string to output file
fout->write(dictionary[ac(curr_char)], dict_len[ac(curr_char)]);
     }

//Definition of ASCII-CODE function

int ac(char any_char)
      {
      if (any_char<0)
         return(256+(int)any_char);
      else
         return((int)any_char);
      }

//Definition of DFC_EXT function (that makes .dfc extension for a filename)
void dfc_ext(void)
{
for (counter=0; outfile[counter] != '.' && outfile[counter] != '\0';
        counter++);                                            // do nothing
        outfile[counter]='\0';
        strcat(outfile, ".dfc");
}

//Definition of EXE_PATH function (determines path for this executable file)
void exe_path(char *ptr)
{
strcpy(exe_path_string, ptr);
for (counter=strlen(exe_path_string)-1;
     counter != 0 && exe_path_string[counter] != '\\'; )
         counter--;
exe_path_string[++counter]='\0';
strcat(exe_path_string, datfile);
}

//========================End of file=========================================
